/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.axis.utils;

import org.apache.axis.MessageContext;
import org.apache.axis.AxisEngine;

/**
 * An utility class to help associate a specific MessageContext with an 
 * arbitrary thread. <BR>
 * Usage:
 * <pre>
 * MessageContextHelper helper = new MessageContextHelper(newCtx);
 * helper.set();
 * try {
 *   // call code that needs MessageContext associated with the thread
 * } finnally {
 *   helper.restore();
 * }
 * </pre>
 */
public class MessageContextHelper {
    
    private MessageContext newCtx;
    private MessageContext oldCtx;
    
    public MessageContextHelper(MessageContext newCtx) {
        this.newCtx = newCtx;
    }
    
    /**
     * Saves the old MessageContext associated with the current thread
     * and associates a new context with the current thread.
     */
    public void set() {
        this.oldCtx = HelperAxisEngine.getCurrentMessageContext();
        HelperAxisEngine.setCurrentMessageContext(this.newCtx);
    }
    
    /**
     * Restores the MessageContext that was previously associated with the
     * current thread.
     */
    public void unset() {
        HelperAxisEngine.setCurrentMessageContext(this.oldCtx);
    }
    
    public static class HelperAxisEngine extends AxisEngine {

        public HelperAxisEngine() {
            super(null);
        }

        public void init() {
        }

        public AxisEngine getClientEngine() {
            return null;
        }

        public void invoke(MessageContext ctx) {
        }

        public static void setCurrentMessageContext(MessageContext mc) {
            AxisEngine.setCurrentMessageContext(mc);
        }
    }

}
