/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.wsrf.container;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Vector;

/**
 * This class is responsible for managing a set of threads.
 * New threads can be added to the pool at any given time.
 */
public class ServiceThreadPool {
    
    private Vector serviceThreads = new Vector();
    protected ServiceDispatcher dispatcher;

    static Log logger = 
        LogFactory.getLog(ServiceThreadPool.class.getName());

    public ServiceThreadPool(ServiceDispatcher dispatcher) {
        this.dispatcher = dispatcher;
    }

    public void startThreads(int threads) {
        for (int i = 0; i < threads; i++) {
            ServiceThread serviceThread = createThread();
            this.serviceThreads.addElement(serviceThread);
            serviceThread.start();
        }
    }

    public int getThreads() {
        return this.serviceThreads.size();
    }

    protected ServiceThread createThread() {
        return new ServiceThread(this.dispatcher);
    }

    public void stopThreads(int numThreads) {
        this.dispatcher.getServiceRequestQueue().stopThreads(numThreads);
    }

    public void stopThreads() {
        stopThreads(getThreads());
    }

    public void removeThread(ServiceThread thread) {
        this.serviceThreads.removeElement(thread);
        synchronized (this) {
            notifyAll();
        }
    }

    public synchronized void waitForThreads() 
        throws InterruptedException {
        while (!isDone()) {
            wait();
        }
    }
    
    public synchronized void waitForThreads(int timeout) 
        throws InterruptedException {
        while (timeout > 0 && !isDone()) {
            wait(5000);
            timeout -= 5000;
        }
    }
    
    private boolean isDone() {
        int size = serviceThreads.size();
        return ((size == 0) || 
                (size == 1 && 
                 serviceThreads.contains(Thread.currentThread())));
    }
    
}
